const uuid = require('uuid/v4');
const ApiResource = require('./api_resource.js');
const Cookies = require('js-cookie');

class Event {

  static create(config, name, params = {}, properties = {}) {
    this._config = config;
    const payload = Object.assign(params, Event._buildPayload(name, params, properties));

    return ApiResource.post(config, this._resourcePath, payload);
  }

  // private

  static set _config(value) {
    this.configuration = value;
  }

  static get _config() {
    return this.configuration;
  }

  static get _resourcePath() {
    return 'events';
  }

  static _buildPayload(name, params, properties) {
    return ({
      name,
      properties,
      message_id: uuid(),
      context: this._context,
      timestamp: this._timestamp,
      identifier_id: params.identifierId || this._defautlIdentifierId,
    });
  }

  static get _context() {
    return ({
      library: {
        name: this._config.libraryName,
        language: this._config.language,
        version: this._config.version,
        snippet_version: this._config.snippetVersion,
      },
    });
  }

  static get _timestamp() {
    return Math.round(Date.now() / 1000);
  }

  static get _defautlIdentifierId() {
    const cookieValue = Cookies.get(this._config.identifierCookieName) || uuid();

    Cookies.set(this._config.identifierCookieName, cookieValue);

    return cookieValue;
  }

}

module.exports = Event;
